﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Infragistics.Controls.Charts;
using System.Collections.ObjectModel;
using System.Collections;

namespace DataServiceGraphTest
{
    public partial class MainPage : UserControl
    {
        const uint _MaxPoints = 1000;

        RandomDataServiceProxy.RandomDataServiceClient client;

        public ObservableCollection<RandomDataServiceProxy.RandomDataModel> Data
        { get; set; }

        public ObservableCollection<RandomDataServiceProxy.RandomDataSummaryModel> SummaryData
        { get; set; }

        public MainPage()
        {
            Data = new ObservableCollection<RandomDataServiceProxy.RandomDataModel>();
            SummaryData = new ObservableCollection<RandomDataServiceProxy.RandomDataSummaryModel>();

            InitializeComponent();

            DataContext = this;

            this.client = new RandomDataServiceProxy.RandomDataServiceClient(
                new PollingDuplexHttpBinding { DuplexMode = PollingDuplexMode.MultipleMessagesPerPoll },
                new EndpointAddress("../RandomDataService.svc"));

            this.client.NotifyRandomDataReceived += new EventHandler<RandomDataServiceProxy.NotifyRandomDataReceivedEventArgs>(client_NotifyRandomDataReceived);
            this.client.NotifySummaryDataReceived += new EventHandler<RandomDataServiceProxy.NotifySummaryDataReceivedEventArgs>(client_NotifySummaryDataReceived);

            client.SubscribeAsync();
        }

        void client_NotifySummaryDataReceived(object sender, RandomDataServiceProxy.NotifySummaryDataReceivedEventArgs e)
        {
            if (e.Error == null)
            {
                foreach (var item in e.summaryData)
                {
                    SummaryData.Add(item);
                }
            }
            else
            {
                this.Messages.Text += string.Format("{0}\n", e.Error.ToString());
            }

            while (SummaryData.Count > ((_MaxPoints / e.summaryData[0].DecimationInterval) - 1))
            {
                SummaryData.RemoveAt(0);
            }

            this.SetMinAndMaxForXAxes();

        }
        private void SetMinAndMaxForXAxes()
        {
            CategoryDateTimeXAxis daxis = this.xmDataChart.Axes["daxis"] as CategoryDateTimeXAxis;
            CategoryDateTimeXAxis xAxis2 = this.xmDataChart.Axes["xAxis2"] as CategoryDateTimeXAxis;

            DateTime minDate1, maxDate1, minDate2, maxDate2;
            this.GetRange(this.SummaryData, out minDate1, out maxDate1);
            this.GetRange(this.Data, out minDate2, out maxDate2);

            DateTime minDate;
            if (minDate1 < minDate2)
            {
                minDate = minDate1;
            }
            else
            {
                minDate = minDate2;
            }
            DateTime maxDate;
            if (maxDate1 > maxDate2)
            {
                maxDate = maxDate1;
            }
            else
            {
                maxDate = maxDate2;
            }

            daxis.MinimumValue = xAxis2.MinimumValue = minDate;
            daxis.MaximumValue = xAxis2.MaximumValue = maxDate;

        }
        private void GetRange(IEnumerable data, out DateTime min, out DateTime max)
        {
            DateTime minDateTime = DateTime.MaxValue;
            DateTime maxDateTime = DateTime.MinValue;
            min = DateTime.MaxValue;
            max = DateTime.MinValue;
            foreach (object o in data)
            {
                DateTime currentDate = DateTime.MinValue;
                if (o is RandomDataServiceProxy.RandomDataModel)
                {
                    RandomDataServiceProxy.RandomDataModel item = (RandomDataServiceProxy.RandomDataModel)o;
                    currentDate = item.Datetime;
                }
                else if (o is RandomDataServiceProxy.RandomDataSummaryModel)
                {
                    RandomDataServiceProxy.RandomDataSummaryModel item = (RandomDataServiceProxy.RandomDataSummaryModel)o;
                    currentDate = item.Datetime;
                }
                
                if (currentDate < min)
                {
                    min = currentDate;
                }
                if (currentDate > max)
                {
                    max = currentDate;
                }
            }
            
        }
        

        void client_NotifyRandomDataReceived(object sender, RandomDataServiceProxy.NotifyRandomDataReceivedEventArgs e)
        {
            if(e.Error == null)
            {
                foreach(var item in e.randomData)
                {
                    Data.Add(item);
                    //this.Messages.Text += string.Format("{0}:{1}\n", item.Datetime.ToString(), item.Value.ToString());
                }
            }
            else
            {
                this.Messages.Text += string.Format("{0}\n", e.Error.ToString());
            }

            while (Data.Count > _MaxPoints)
            {
                Data.RemoveAt(0);
            }
            this.SetMinAndMaxForXAxes();
        }
    }
}
